/*
    busmonitor.h - Busmonitor for EIB PEI16 Linux Driver

    Copyright (C) 2006 Ulrich Krenn <e0305817@stud4.tuwien.ac.at>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/


#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <sys/un.h>
#include <unistd.h>
#include <fcntl.h>
#include <stropts.h>    /* ioctl */
#include "eib-user.h"    /* ioctl definitions .h file */



#define VERSION                             "0.5"

#define STANDARDDEVICE                      "/dev/eib0"

#define MAX_MSG_LEN                         32

#define IOCTL_VERSION                       EIB_IOCTL_VERSION

#define S_IN_BUFF_LEN                       3

#define ALARM_TIME                          10
#define ALARM_CANT_WRITE_TO_DRIVER           0
#define ALARM_BAU_DOESNT_ANSWER             1


#define MASK_BIT0                           0x01
#define MASK_BIT1                           0x02
#define MASK_BIT2                           0x04
#define MASK_BIT3                           0x08
#define MASK_BIT4                           0x10
#define MASK_BIT5                           0x20
#define MASK_BIT6                           0x40
#define MASK_BIT7                           0x80




typedef struct{
    unsigned char       m_code;
    unsigned char       status;
    unsigned short      timeStamp;
    unsigned char       control;
    unsigned char       srcAddrHigh;
    unsigned char       srcAddrLow;
    unsigned char       destAddrHigh;
    unsigned char       destAddrLow;
    unsigned char       byte10;
    unsigned char       byte11;
    unsigned char       byte12;
}T_BUSMON_MSG;



#define print_flush(str){\
    if (fprintf(stdout, str) < 0)\
        (void)bailOut("Can't write to stdout!");\
    if (fflush(stdout) == EOF)\
        (void)bailOut("Error fflush");\
}\

#define print_flush_v(str, val){\
    if (fprintf(stdout, str, val) < 0)\
        (void)bailOut("Can't write to stdout!");\
    if (fflush(stdout) == EOF)\
        (void)bailOut("Error fflush");\
}\


#define bus_print(str){\
    if (fprintf(stdout, str) < 0)\
        (void)bailOut("Can't write to stdout!");\
}\

#define bus_print_v(str, val){\
    if (fprintf(stdout, str, val) < 0)\
        (void)bailOut("Can't write to stdout!");\
}\


#define bus_flush(){\
    if (fflush(stdout) == EOF)\
        (void)bailOut("Error fflush");\
}\




#define F_RECEIVE(code){\
    bus_alarm(10, ALARM_BAU_DOESNT_ANSWER); \
    while(1){\
        if ((len = read(eib_fd, (unsigned char*)&recvBuff, MAX_MSG_LEN)) != -1){\
            recvBuff[len] = 0;\
            if (len != 0){\
                code;\
            }\
            else{\
                print_flush("Message len: 0 \n");\
            }\
        }\
        else{\
            print_flush_v("Receive Status %d\n", len);\
        }\
    }\
    alarm(0);\
}\


