/*
    eib-user.h - IOCTL definitions for serial EIB PEI16 Linux Device Driver

    Copyright (C) 2006 Ulrich Krenn <e0305817@stud4.tuwien.ac.at>
    Copyright (C) 2006 Martin Koegler <mkoegler@auto.tuwien.ac.at>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/


/* File version V0.1 */

#ifndef __EIB_USER__
#define __EIB_USER__
  
/* kernel or user space */
#ifdef __KERNEL__
    #include <linux/types.h>
#else
    #include <stdint.h>
#endif


/* 2.4 device statistic */
struct eib_device_information {
    int8_t        string[255];
    uint32_t    warning_flag;
    int32_t        rb_length;
    int32_t        wb_length;
    int32_t        rb_max_length;
    int32_t        wb_max_length;
    uint32_t    stat_rx_bytes;
    uint32_t    stat_tx_bytes;
    uint32_t    stat_rx_data_blocks;
    uint32_t    stat_tx_data_blocks;
    uint32_t    stat_max_rx_time_data_block;
    uint32_t    stat_max_tx_time_data_block;
    int32_t        io;
    int32_t        irq;
    int32_t        major;
};


/* eib-pei16-2.6.15-1.0 driver statistics */
struct eib_driver_statistic_v1{
    uint32_t    comResetCnt;        /* number of resets the driver carried out */
    uint32_t    resetMsgCnt;        /* number reset messages from the BAU */
    uint32_t    overrunErrorCnt;    /* number overrun errors */
    uint32_t    msgTimoutCnt;        /* number of message timouts */
    uint32_t    initSendTimeoutCnt;    /* no response to setRTS */
    uint32_t    unexpectedIntCnt;    /* unexpected interrupt */
    uint32_t    wrongAckCnt;        /* wrong acknowledge, not 0x00 */

    uint32_t    timeOpened;            /* time when open() finished */
    uint32_t    timeLastReset;        /* time when com reset was called */
    uint32_t    timeStatistics;        /* time when statistics where requeseted */

    uint32_t    framingErrorCnt;    /* number framing errors */
    uint32_t    parityErrorCnt;        /* number parity errors */
    uint32_t    recvForcedCnt;        /* number of times the BAU forced the driver to read a message */
    uint32_t    recvBuffFullCnt;    /* number of times BAU tried to send but read buffer full */
    uint32_t    timeoutTEMT_Cnt;    /* number of times that a TEMT timeout happend */
    uint32_t    msgSentCnt;            /* number messages sent */
    uint32_t    bytesSentCnt;        /* number bytes sent */
    uint32_t    msgReceivedCnt;        /* number messages received */
    uint32_t    bytesReceivedCnt;    /* number bytes received */

    uint16_t    messagesInSendBuffer;    /* actual number messages in send buffer */
    uint16_t    messagesInReceiveBuffer;/* actual number messages in receive buffer */
 
    uint8_t        lastSendMsgLen;        /* last sent message length */
    uint8_t        lastRecvMsgLen;        /* last received message length */

    uint16_t    port;                /* port number of the instance */
    uint32_t    irq;                /* interrupt line number of the instance */

};


/* magic number */
#define EIB_IOCTL_MAGIC                ('E'+'I'+'B')    /* 0xd0 */

/* in all versions */
#define EIB_IOCTL_VERSION                _IO (EIB_IOCTL_MAGIC, 0)
#define EIB_IOCTL_RES_STATISTICS        _IO (EIB_IOCTL_MAGIC, 2)
#define EIB_IOCTL_RESET_WRITE_BUFFER    _IO (EIB_IOCTL_MAGIC, 3)
#define EIB_IOCTL_RESET_READ_BUFFER        _IO (EIB_IOCTL_MAGIC, 5)
    /* only this 3 ioctl commands exist in all driver versions (2.4, 2.6, eib-pei16-2.6.15-x.x)
    The EIB_IOCTL_VERSION is split up into 3 sections XXYYZZZZhex
    XX:    Numbering schema (00).
    YY: Development lines (00 = driver versions 2.4 and 2.6; 01 = eib-pei16-2.6.15-x.x).
    ZZZZ: Interface version.
    */

/* in 2.4. */
#define EIB_GET_DEVICE_INFORMATION      _IOR(EIB_IOCTL_MAGIC, 1, struct eib_device_information)
#define EIB_SINGLE_OPEN                 _IO (EIB_IOCTL_MAGIC, 4)

/* in eib-pei16-2.6.15-x.x (API version 0100) */
#define EIB_IOCTL_RESET                 _IO (EIB_IOCTL_MAGIC, 6)
#define EIB_IOCTL_DEBUG_ON              _IO (EIB_IOCTL_MAGIC, 7)
#define EIB_IOCTL_DEBUG_OFF             _IO (EIB_IOCTL_MAGIC, 8)
#define EIB_IOCTL_STATISTICS            _IOR (EIB_IOCTL_MAGIC, 9, struct eib_driver_statistic_v1)
#define EIB_IOCTL_STATUS                _IO (EIB_IOCTL_MAGIC, 10)
#define EIB_IOCTL_RESET_MSG_ON          _IO (EIB_IOCTL_MAGIC, 11)
#define EIB_IOCTL_RESET_MSG_OFF         _IO (EIB_IOCTL_MAGIC, 12)

/* Compat */
#define EIB_GET_BCU_VERSION             EIB_IOCTL_VERSION
#define EIB_CLEAR_WRITE_BUFFER          EIB_IOCTL_RESET_WRITE_BUFFER       
#define EIB_CLEAR_READ_BUFFER           EIB_IOCTL_RESET_READ_BUFFER 
#define EIB_RESET_DEVICE_STATISTIC      EIB_IOCTL_RES_STATISTICS


#endif /* #ifndef __EIB_USER__ */

