
/*
    groupWrite.c - Program for EIB PEI16 Linux Driver to send group messages

    Copyright (C) 2006 Ulrich Krenn <e0305817@stud4.tuwien.ac.at>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/


/**** includes ****/
#include <linux/ioctl.h>    /* _IO */
#include "groupwrite.h"


/**** functions ****/
void freeResources(void);
void bailOut(char *message);
int getAddress(unsigned char *cha);
void signalHandler(int sig);
void gw_alarm(int time, int status);
void f_send(char* str, int len);
void f_fsync(int t);

    
/**** global variables ****/
char            *device;                                /* deivce name */
int             eib_fd,address[3],alarm_status;         /* filedescriptor, addr(1/1/1, bits 5-3-8) */
unsigned char   statusBAU;


/**** main ****/
int main(int argc, char **argv){
    int                 i=0,len=0,mode=0,strLen=0,bitStatus=0,lenSend=0;
    char                inBuff[S_IN_BUFF_LEN+1];        /* +1 nullterm. */
    unsigned long       vers=0;
    unsigned char       ok=0, msg[MAX_MSG_LEN],recvBuff[MAX_MSG_LEN],tmpCha=0;
    unsigned short      tmpSho=0;

    memset(&inBuff[0], 0, sizeof(inBuff));
    memset(&msg[0], 0, sizeof(msg));

    /**** initialization ****/
    argc = argc;
    argv = argv;
    device = STANDARDDEVICE;
    alarm_status = 0;
    eib_fd = -1;
    statusBAU = 0;
    memset(&address[0], 0, sizeof(address));

    /* fixed values in the message */
    msg[0] = 0x11;    /* byte 1: m_code */
    msg[1] = 0x00;    /* byte 2: control field 00,0C? */
    msg[2] = 0x00;    /* byte 3: unused */
    msg[3] = 0x00;    /* byte 4: unused */
    msg[6] = 0xFF;    /* byte 7: - */
    msg[7] = 0x00;    /* byte 8: - */
    msg[8] = 0x00;    /* byte 9: - */

    /* signaling */
    signal(SIGINT, signalHandler);              /* ctrl-c */
    signal(SIGALRM, signalHandler);             /* alarm() timeout */

    /**** print intro ****/
    print_flush("\n");
    print_flush("=======================================\n");
    print_flush_v("          Group Write V %s \n", VERSION);
    print_flush("=======================================\n\n");

    print_flush("Press Strg-C to end programm.\n\n");

    /**** open driver *** */
    print_flush_v("Opening: %s\n\n", device);
    if ((eib_fd = open(device, O_RDWR)) == -1){        /* open file descriptor */
        bailOut("Can't open");
    }


    /**** get driver interface version number ****/
    vers = ioctl(eib_fd, IOCTL_VERSION);
    if (fprintf(stdout, "Driver interface version: %X\n\n", (unsigned int)vers) < 0)
        (void)bailOut("Can't write to stdout!");
    gw_flush();
    if (vers != 0x00010010){
        (void)bailOut("Wrong driver interface version!");
    }



    /**** request BAU mask version ****/
    print_flush("Requesting BAU mask version: ");
    f_send("\x4C\x02\x00\x4E", 4);
    f_fsync(ALARM_TIME);


    /* receive confirmation */
    F_RECEIVE(
        if ((len == 6) && (recvBuff[0] == 0x4B)){                /* PC_GET_VAL.con */
            if ((recvBuff[1] == 0x02) && (recvBuff[2] == 0x00) && (recvBuff[3] == 0x4E)){
                *((unsigned char*)&tmpSho) = recvBuff[5];
                *((unsigned char*)&tmpSho + 1) = recvBuff[4];    /* may don't work on little endien */
                break;
            }
        }
    )/* macro */

    /* Mask Type (8 bits)               Firmware Version (8 bits)       */
    /* Medium Type    Firmware Type     Version         Sub-code        */
    /* MMMM           TTTT              VVVV            SSSS            */
    tmpCha = 0;        /* set 1 if BCU1 */
    switch (tmpSho){
        case 0x0010:
            gw_print(" Device= BCU 1, VIM M1111 | Medium= Twisted Pair 1 | Vers.= 0");
            tmpCha = 1;
            break;
        case 0x0011:
            gw_print(" Device= BCU 1, VIM M1111 | Medium= Twisted Pair 1 | Vers.= 1");
            tmpCha = 1;
            break;
        case 0x0012:
            gw_print(" Device= BCU 1, VIM M1111 | Medium= Twisted Pair 1 | Vers.= 2");
            tmpCha = 1;
            break;
        case 0x1012:
            gw_print(" Device= BCU 1 | Medium= Power Line 110 | Vers.= 2");
            tmpCha = 1;
            break;
        case 0x1013:
            gw_print(" Device= BCU 1 | Medium= Power Line 110 | Vers.= 3");
            tmpCha = 1;
            break;
        case 0x0020:
            gw_print(" Device= BCU 2, BIM M113 | Medium= Twisted Pair 1 | Vers.= 0");
            break;
        case 0x0021:
            gw_print(" Device= BCU 2 | Medium= Twisted Pair 1 | Vers.= 1");
            break;
        case 0x0700:
            gw_print(" Device= BIM M112 | Medium= Twisted Pair 1 | Vers.= 0");
            break;
        case 0x0701:
            gw_print(" Device= BIM M112 | Medium= Twisted Pair 1 | Vers.= 1");
            break;
        case 0x0810:
            gw_print(" Device= IR-Decoder | Medium= Twisted Pair 1 | Vers.= 0");
            break;
        case 0x0811:
            gw_print(" Device= IR-Decoder | Medium= Twisted Pair 1 | Vers.= 1");
            break;
        case 0x0910:
            gw_print(" Device= Coupler 1.0 | Medium= Twisted Pair 1 | Vers.= 0");
            break;
        case 0x0911:
            gw_print(" Device= Coupler 1.1 | Medium= Twisted Pair 1 | Vers.= 1");
            break;
        case 0x1900:
            gw_print(" Device= Media Coupler PL-TP | Medium= Twisted Pair 1 | Vers.= 0");
            break;
        case 0x3012:
            gw_print(" Device= BCU 1 | Medium= Twisted Pair 0 | Vers.= 2");
            tmpCha = 1;
            break;
        case 0x4012:
            gw_print(" Device= BCU 1 | Medium= Power Line 132 | Vers.= 2");
            tmpCha = 1;
            break;
        default:
            bailOut(" unknowen mask version!");
            break;
    }/* switch */
    print_flush("\n");



    /**** request BAU mode */
    print_flush("Request BAU mode: ");
    f_send("\x4C\x01\x00\x60", 4);
    f_fsync(ALARM_TIME);

    F_RECEIVE(
        if ((len == 5) && (recvBuff[0] == 0x4B)){                /* PC_GET_VAL.con */
            statusBAU = recvBuff[4];
            print_flush_v("%X\n", recvBuff[4]);
            break;
        }
    )/* macro */


    /**** switch to link layer mode ****/
    print_flush("Switching to link layer mode.\n");
    f_send("\x46\x01\x00\x60\x12", 5);
    f_fsync(ALARM_TIME);


    /**** request local address ****/
    /* request BAU address (PC_GET_VAL.req, 0117-0118) */
    print_flush("Requesting local address: ");
    f_send("\x4C\x02\x01\x17", 4);
    f_fsync(ALARM_TIME);

    /* receive confirmation */
    /* print_flush("Waiting for BAU response \n\n"); */
    F_RECEIVE(
        if ((len == 6) && (recvBuff[0] == 0x4B)){    /* PC_GET_VAL.con */
            if ((recvBuff[1] == 0x02) && (recvBuff[2] == 0x01) && (recvBuff[3] == 0x17)){
                gw_print_v(" %d, ", ((recvBuff[4] & 0xF0) >> 4));   /* XXXxxxxx xxxxxxxx */
                gw_print_v("%d, ", (recvBuff[4] & 0x0F));           /* xxxXXXXX xxxxxxxx */
                gw_print_v("%d\n", recvBuff[5]);                    /* xxxxxxxx XXXXXXXX */
                gw_flush();
                break;
            }
        }
    )/* macro */

    /**** user input ****/
    /* read group address */
    do{
        ok = 0;
        print_flush("\nGroup address: ");
        if ((fgets(inBuff, S_IN_BUFF_LEN, stdin)) != NULL){
            if (getAddress((unsigned char*)&inBuff[0]) == 0){
                ok = 1;
            }
            else{
                print_flush("wrong address!\n");
            }
        }
        if(ferror(stdin))
            bailOut("Error: reading from stdin!");

    }while (ok == 0);


    /* read message type */
    do{
        print_flush("Choose message (1=bit, 2=string): ");
        if ((fgets(inBuff, S_IN_BUFF_LEN, stdin)) != NULL){
            mode = strtol(inBuff, NULL, 0);
        }
        if(ferror(stdin))
            bailOut("Error: reading from stdin!");

        if ((mode != 1) && (mode != 2)){
            print_flush("Wrong Input!\n\n");
        }

    }while ((mode != 1) && (mode != 2));


    /* destination address */
    msg[4] = (address[2] << 3) | address[1];    /* byte 5: dest adr. */
    msg[5] = address[0];                        /* byte 6: dest adr. device */

    /* switch message type */
    switch(mode){

        case 1:            /* group write 1 bit (switch device on/off) */

            /* read bit status */
            do{
                print_flush("Bit status (0,1): ");
                if ((fgets(inBuff, S_IN_BUFF_LEN, stdin)) != NULL){
                    bitStatus = strtol(inBuff, NULL, 0);
                }
                if(ferror(stdin))
                    bailOut("Error: reading from stdin!");
    
                if ((bitStatus != 0) && (bitStatus != 1)){
                    print_flush("Wrong Input!\n\n");
                }

            }while ((bitStatus != 0) && (bitStatus != 1));

            if (bitStatus == 0){
                msg[8] = 0x80;    /* switch on */
            }
            else{
                msg[8] = 0x81;    /* switch off */
            }
            lenSend = 9;
            break;


        case 2:            /* send string */

            /* read string */
            do{
                print_flush("String (length 0-14): ");
                if ((fgets(inBuff, S_IN_BUFF_LEN, stdin)) != NULL){
                    strLen = strlen(inBuff);
                    inBuff[strLen] = 0;
                    strLen--;    /* cr */
                    if ((strLen < 0) || (strLen > 14))
                        print_flush("Input not valid!\n");
                }
                if(ferror(stdin))
                    bailOut("Error: reading from stdin!");
    
            }while ((strLen < 0) || (strLen > 14));

            msg[6] = strLen;
            msg[8] = 0x80;

            for (i=0; i<strLen; i++)
                msg[i+9] = inBuff[i];

            lenSend = 9 + strLen;
            break;

        default:
            bailOut("Wrong mode!");
            break;
    }


    print_flush("Sending: ");
    for (i=0; i<lenSend; i++){
        print_flush_v("%X ", msg[i]);
    }
    print_flush("\n\n");

    f_send((char*)&msg[0], lenSend);
    f_fsync(ALARM_TIME);

    print_flush("Receiving confirmation: \n");
    F_RECEIVE(
        /* check for response (same message back) */
        if (lenSend == len){
            unsigned char diff=4;        /* first 4 bytes are different (1 x m_code, 1 x control, 2 x src addr) */

            if (recvBuff[0] == 0x4E){    /* data confirmation */
                print_flush("Confirmation received.\n");

                if (memcmp((void*)&msg[diff], (void*)&recvBuff[diff], (lenSend - diff)) != 0){
                    print_flush("Confirmation message different to sent message!\n");
                }

                if (recvBuff[1] & 0x01){
                    print_flush(" (confirm error!\n)");
                }
                gw_print_v("len: %d; ", len);
                gw_print("msg: ");
                for (i=0; i<len; i++){ 
                    gw_print_v("%X ", recvBuff[i]);
                }
                print_flush("\n");

                break;

            }/* memcmp */
        }/* same length */
    )/* macro */

    print_flush("Done!\n");

    return 0;

}/* main */



/* bailOut */
void bailOut(char *message){

    alarm(0);
    if (errno != 0){
        fprintf(stdout, "\nError: %s (strerror: %s) \n\n", message, strerror(errno));
    }
    else{
        fprintf(stdout, "\nError: %s \n\n", message);
    }
    freeResources();
    exit(-1);

}/* bailOut */


/* freeResources */
void freeResources(void){
    if (eib_fd != -1) {
        close(eib_fd);
        eib_fd = -1;
    }
}


/* get the 3 address value out of the string xx/x/xxx, or x/x/x or,... */
int getAddress(unsigned char *cha){
    int len=0,i=0;
    unsigned char    dot1Pos=0, dot2Pos=0, nr[4];

    len = strlen((char*)cha);
    cha[len] = 0;                           /* clear line feed */
    len--;
    
    if (len < 5)                            /* to short, min x/x/x */
        return -1;

    for (i=0; i<len; i++){                  /* dot pos 1 */
        if ((cha[i] == '.') || (cha[i] == '/')){
            dot1Pos = i;
            break;
        }
    }

    for (i=(dot1Pos + 1); i<len; i++){      /* dot pos 2 */
        if ((cha[i] == '.') || (cha[i] == '/')){
            dot2Pos = i;
            break;
        }
    }

    if (dot1Pos >= dot2Pos)                 /* error dots */
        return -1;
    if ((dot1Pos == 0) || (dot1Pos > 2))    /* 1-2 (number digits of values) */
        return -1;
    if ((dot2Pos - dot1Pos - 1) != 1)       /* 1 */
        return -1;
    if (((len - 1 - dot2Pos) == 0) || ((len - 1 - dot2Pos) > 3)) /* 1-3 */
        return -1;

    memset(&nr[0], 0, sizeof(nr));          /* get first value */
    for (i=0; i<dot1Pos; i++){
        nr[i] = cha[i];
        if ((nr[i] < '0') || (nr[i] > '9')) return -1;
    }
    address[2] = strtol((char*)nr, NULL, 0);

    memset(&nr[0], 0, sizeof(nr));          /* get second value */
    for (i=0; i<(dot2Pos - dot1Pos - 1); i++){
        nr[i] = cha[i+(dot1Pos + 1)];
        if ((nr[i] < '0') || (nr[i] > '9')) return -1;
    }
    address[1] = strtol((char*)nr, NULL, 0);

    memset(&nr[0], 0, sizeof(nr));          /* get third value */
    for (i=0; i<(len - dot2Pos - 1); i++){
        nr[i] = cha[i+(dot2Pos + 1)];
        if ((nr[i] < '0') || (nr[i] > '9')) return -1;
    }
    address[0] = strtol((char*)nr, NULL, 0);

    if (address[2] > 31) return -1;         /* limit values */
    if (address[1] > 7) return -1;
    if (address[0] > 255) return -1;

    return 0;

}/* get adress */


/* signalHandler */
void signalHandler(int sig){
    unsigned char    msg[5] = {0x46, 0x01, 0x00, 0x60, statusBAU};

    switch (sig){

        case SIGINT:                     /* ctrl-c */
        case SIGTERM:                    /* termination signal */

            print_flush("Restoring BAU mode\n\n");
            f_send((char*)&msg[0], 5);   /* restoring BAU mode (PC_SET_VAL.req) */

            sleep(5);

            freeResources();
            exit(0);
            break;

        case SIGALRM:

            switch (alarm_status){
                case ALARM_CANT_WRITE_TO_DRIVER:
                    fprintf(stdout,"Can't send message to BAU!\n\n");
                    break;
                case ALARM_BAU_DOESNT_ANSWER:
                    fprintf(stdout,"BCU doesn't answer!\n\n");
                    break;
                default:
                    fprintf(stdout,"Unknown Error!\n\n");
                    break;
            }
            fflush(stdout);

            freeResources();
            exit(-1);
            break;

    }/* switch */

}/* signalHandler */



/* timeout + status */
void gw_alarm(int time, int status){
    alarm(time);
    alarm_status = status;
}/* gw_alarm */


/* wait some time till message is sent */
void f_fsync(int t){
    gw_alarm(t, ALARM_CANT_WRITE_TO_DRIVER);
    if (fsync(eib_fd) != 0){
        alarm(0);
        fprintf(stdout,"Can't send message to BAU!\n");
        fflush(stdout);
        freeResources();
        exit(-1);
    }
    alarm(0);
}/* f_fsync */


void f_send(char* str, int len){
    gw_alarm(ALARM_TIME, ALARM_CANT_WRITE_TO_DRIVER);    /* timeout */
    if (write(eib_fd, str, len) == -1){      /* (PC_GET_VAL.req) */
        bailOut("Can't write to device");
    }
    alarm(0);                                /* stop timeout */
}/* f_send */

